#!/usr/bin/perl -wT
#
# URLblocked.cgi - explain to the user that the URL is blocked and by which rule set.
#
# Currently the error messages supports 
# en (English), 
# de (German), 
# pl (Polish)
# sv (Swedisk)
# it (Italian)
# pt (Portuguese)
# fr (French)
# tr (Turkish)
# nl (Dutch).
# You can add a language yourself: search for all occurences of "NEWLANGUAGE"
# and add your language text.

use strict;
use Socket;
# use FCGI;		### without FCGI this statement is not necessary

use constant {
   CT_IMAGE  => 1,
   CT_JAVA   => 2,
   CT_HTML   => 3,
   CT_XML    => 4,
   CT_CSS    => 5,
   CT_TEXT   => 6,
   CT_JSON   => 7,
   CT_STREAM => 8
};

use vars qw( $admin $clientaddr $clientname $clientuser $clientgroup $targetgroup );
use vars qw( $color $size $mode $textcolor $bgcolor $titlesize $textsize $url );
use vars qw( @day @month @languages $lang $protocol $address $port $path );

local $admin;
local $clientaddr;
local $clientname;
local $clientuser;
local $clientgroup;
local $targetgroup;
local $color;
local $size;
local $mode;
local $textcolor;
local $bgcolor;
local $titlesize;
local $textsize;
local $url;
local $lang;
local $protocol;
local $address;
local $port;
local $path;
local @day = /usr/share/ufdbGuard;
local @month = /usr/share/ufdbGuard;
local @languages = (
		"de (German),",
		"nl (Dutch),",
		"pl (Polish),",
		"sv (Swedish),",
		"es (Spanish),",
		"it (Italian),",
		"pt (Portuguese),",
		"fr (French),",
		"tr (Turkish),",
		"NEW (NEWLANGUAGE),",
		"en (English),",
	       );

my $html_comment = "<!-- this is a very long comment to signal
to MSIE and other browsers not to display their own 'user-friendly'
error message, but to display the one that is produced by this
program.\n
The rest is this message is bogus to make it longer
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx  -->\n";

sub init();
sub session_reinit();
sub getPreferedLanguage(@);
sub parseURL($);
sub parseQuery($);


sub init () 
{
   ( $admin, $clientaddr, $clientname, $clientuser, $clientgroup, $targetgroup, $url )  =
      parseQuery( $ENV{"QUERY_STRING"} );
   $lang = /usr/share/ufdbGuard;
}


sub session_reinit ()
{
   $admin = /usr/share/ufdbGuard;
   $clientaddr = /usr/share/ufdbGuard;
   $clientname = /usr/share/ufdbGuard;
   $clientuser = /usr/share/ufdbGuard;
   $clientgroup = /usr/share/ufdbGuard;
   $targetgroup = /usr/share/ufdbGuard;
   $color = /usr/share/ufdbGuard;
   $size = /usr/share/ufdbGuard;
   $mode = /usr/share/ufdbGuard;
   $url = /usr/share/ufdbGuard;
   $lang = /usr/share/ufdbGuard;
}


#
# Find the first supported language of the client.
#
sub getPreferedLanguage (@) 
{
  my @supported = /usr/share/ufdbGuard;
  my @clientLanguages = /usr/share/ufdbGuard;
  my $lang;
  my $supp;

  ### NO!  push(@clientLanguages,$supported[0]);
  for $lang (@clientLanguages) 
  {
    $lang =~ s/\s.*//;
    $lang =~ s/-.*//;
    for $supp (@supported) 
    {
      $supp =~ s/\s.*//;
      return($lang) if ($lang eq $supp);
    }
  }
  return 'en';   # default language is 'en'
}


sub parseQuery ($) 
{
  my $query       = /usr/share/ufdbGuard;
  my $admin       = /usr/share/ufdbGuard;
  my $clientaddr  = /usr/share/ufdbGuard;
  my $clientname  = /usr/share/ufdbGuard;
  my $clientuser  = /usr/share/ufdbGuard;
  my $clientgroup = /usr/share/ufdbGuard;
  my $targetgroup = /usr/share/ufdbGuard;
  my $url         = /usr/share/ufdbGuard;

  if (defined($query))
  {
    while ($query =~ /^\&?([^\&=]+)=\"([^\"]*)\"(.*)/  || 
           $query =~ /^\&?([^\&=]+)=([^\&=]*)(.*)/)
    {
      my $key = /usr/share/ufdbGuard;
      my $value = /usr/share/ufdbGuard;
      $value = /usr/share/ufdbGuard;
      $query = /usr/share/ufdbGuard;

      if ($key =~ /^(admin|clientaddr|clientname|clientuser|clientident|clientgroup|category|targetgroup|color|size|source|srcclass|targetclass|mode|url)$/) 
      {
	$value =~ s/%20/ /g;
	$value =~ s/%22/"/g;
	$value =~ s/%23/#/g;
	$value =~ s/%24/\$/g;
	$value =~ s/%26/\&/g;
	$value =~ s/%27/\?/g;
	$value =~ s/%2B/\+/ig;
	$value =~ s/%2C/,/ig;
	$value =~ s/%2F/\//ig;
	$value =~ s/%3A/:/ig;
	$value =~ s/%3B/;/ig;
	$value =~ s/%3C/</ig;
	$value =~ s/%3D/=/ig;
	$value =~ s/%3E/>/ig;
	$value =~ s/%3F/?/ig;
	$value =~ s/%40/\@/ig;
	$value =~ s/%5C/\\/ig;
	$value =~ s/%25/%/g;		# must be last
	$key = /usr/share/ufdbGuard;
	$key = /usr/share/ufdbGuard;
	$key = /usr/share/ufdbGuard;
	eval "\$$key = /usr/share/ufdbGuard;
      }

      if ($query =~ /^url=(.*)/) 
      {
	$url = /usr/share/ufdbGuard;
	last;
      }
    }
  }

  return ( $admin, $clientaddr, $clientname, $clientuser, $clientgroup, $targetgroup, $url );
}


sub parseURL ($) 
{
  my $url      = /usr/share/ufdbGuard;
  my $protocol = /usr/share/ufdbGuard;
  my $address  = /usr/share/ufdbGuard;
  my $port     = /usr/share/ufdbGuard;
  my $path     = /usr/share/ufdbGuard;

  $url =~ /^([^\/:]+):\/\/([^\/:]+)(:\d*)?(.*)/;
  $protocol = /usr/share/ufdbGuard;
  $address  = /usr/share/ufdbGuard;
  $port     = /usr/share/ufdbGuard;
  $path     = /usr/share/ufdbGuard;

  return ( $protocol, $address, $port, $path );
}


sub getContentType( $ )
{
   my $url = /usr/share/ufdbGuard;
   my $suffix;
   my $path;

   $url =~ s/[;\?\&].*//;
   $url =~ s/^(ftp|http|https):\/\///;

   $path = /usr/share/ufdbGuard;
   $path =~ s/^[^\/]*//;

   $suffix = /usr/share/ufdbGuard;
   $suffix =~ s/.*\././;

   return CT_IMAGE  if ($suffix =~ /\.(bmp|gif|ico|jpg|jpeg|jpe|png|tiff)$/i);
   return CT_CSS    if ($suffix =~ /\.css$/i);
   return CT_JSON   if ($suffix =~ /\.json$/i);
   return CT_JAVA   if ($suffix =~ /\.(js|jar)$/i);
   return CT_TEXT   if ($suffix =~ /\.(csv|txt)$/i);
   return CT_HTML   if ($suffix =~ /\.(htm|html|dhtml|shtml)$/i);
   return CT_XML    if ($suffix =~ /\.(xml|sxml|rss)$/i);
   return CT_STREAM if ($suffix =~ /\.(bin|bz2|cab|class|dat|doc|gz|h264|mp3|mpg|mpeg|msi|mst|ppt|pdf|rar|tar|ttf|xls|zip|ogv|divx|xvid|qt|ra|ram|rv|wmv|avi|mov|swf|mp4|mv4|flv)$/i);

   # no suffix found, now we start with the guesswork


   return CT_HTML   if ($path eq '/' || $path eq '');
   return CT_IMAGE  if ($url =~ /^googleadservices\.com\/pagead\/conversion\// );
   return CT_TEXT   if ($url =~ /^googleads\.g\.doubleclick\.net\/pagead\/ads\// );
   return CT_JAVA   if ($url eq 'a.analytics.yahoo.com/fpc.pl' );
   return CT_IMAGE  if ($url eq 'a.analytics.yahoo.com/p.pl' );
   return CT_IMAGE  if ($url eq 'ping.chartbeat.net/ping' );

   if ($url =~ '^www\.youtube\.com')
   {
      return CT_STREAM  if ($path =~ /^\/cp\//  ||  $path =~ /^\/p\//  ||
                            $path =~ /^\/v\//   ||  $path =~ /^\/videoplayback/ );
   }

   return CT_IMAGE  if ($url =~ /^b\.scorecardresearch\.com\// );

   return CT_JAVA   if ($url =~ /\.doubleclick\.net\/adj\//   ||
                        $url =~ /\.doubleclick\.net\/pfadj\// );
   return CT_IMAGE  if ($url =~ /\.doubleclick\.net\/imp/ );

   if ($url =~ /^view\.atdmt\.com\//)
   {
      return CT_IMAGE  if ($path =~ /^\/action\// );
      return CT_JAVA   if ($path =~ /^\/jview\// );
   }

   return CT_JAVA   if ($url eq 'static.ak.connect.facebook.com/connect.php' );

   return CT_IMAGE  if ($url eq 'secure-us.imrworldwide.com/cgi-bin/m' );

   return CT_IMAGE  if ($url =~ /ftjcfx\.com\/image-/ );
   return CT_IMAGE  if ($url =~ /lduhtrp\.net\/image-/ );
   return CT_IMAGE  if ($url =~ /img\.pheedo\.com\/img\.phdo/ );
      
   if ($path =~ /\/realmedia\/ads\//i )
   {
      return CT_JAVA   if ($path =~ /\/adstream_jx/  ||  $path =~ /\/adstream_mjx/ );
      return CT_IMAGE  if ($path =~ /\/adstream_lx/  ||  $path =~ /\/adstream_nx/ );
      return CT_IMAGE  if ($path =~ /\/ads\/cap\.cgi/  );
   }

   return CT_JAVA   if ($url =~ /overture\.com\/ls_js_/ );

   return CT_IMAGE  if ($path =~ /\/scripts\/beacon\.dll/  ||  $path =~ /\/scripts\/beacon2\.dll/ );

   return CT_JAVA   if ($path =~ /\/javascript\//  ||  $path =~ /\/ajaxpro\// );

   return CT_JAVA   if ($path =~ /\/js\.php$/  ||  $path =~ /\/javascript\.php$/ );

   return CT_CSS    if ($path =~ /\/css\.php$/ );

   return CT_IMAGE  if ($path =~ /\/image\.php$/  || $path =~ /\/image\.php\// );

   return CT_JAVA   if ($path =~ /\/js\.ng\//  ||  $path =~ /\/js\// );

   return CT_JAVA   if ($path =~ /\/scripts\//  ||  $path =~ /\/script\// );

   return CT_XML    if ($url =~ /^xml\./ );

   if ($path =~ /\/b\/ss\// )
   {
      return CT_IMAGE  if ($path =~ /\/FAS/i  ||  $path =~ /\/H\./i  ||  $path =~ /\/G\./i );
   }
  
   return CT_JAVA   if ($url =~ /\.channel\.facebook\.com\/x\// );
   return CT_TEXT   if ($url =~ /\.channel\.facebook\.com\/p/ );
   return CT_IMAGE  if ($url eq 'www.facebook.com/fr/u.php' );

   return CT_IMAGE  if ($url eq 'pixel.mathtag.com/event/img' );
   return CT_JAVA   if ($url eq 'pixel.mathtag.com/event/js' );

   return CT_IMAGE  if ($url eq 'x.bidswitch.net/ul_cb/sync' );

   return CT_XML    if ($path =~ /\/xml-rpc/ );

   return CT_STREAM if ($path eq 'open/1');

   return CT_IMAGE  if ($url =~ /^pixel\./  ||  $path =~ /\/pixel$/ );

   return CT_HTML;
}


# while (FCGI::accept() >= 0)
{
   my $time = /usr/share/ufdbGuard;
   my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst);
   my $root;
   my $buffer;

   session_reinit();
   init();

   # the default is english
   my $forbidden = /usr/share/ufdbGuard;
   my $title = /usr/share/ufdbGuard;
   my $explanation_prefix = /usr/share/ufdbGuard;
   my $explanation_suffix = /usr/share/ufdbGuard;
   my $go_back = /usr/share/ufdbGuard;
   my $more_info = /usr/share/ufdbGuard;

   my $contentType = /usr/share/ufdbGuard;

   if ($contentType == CT_IMAGE)
   {
      print "Content-Type: image/png\n";
      ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = /usr/share/ufdbGuard;
      printf "Expires: %s, %02d-%s-%04d %02d:%02d:%02d GMT\n", 
	     $day[$wday], $mday, $month[$mon], $year+1900, $hour, $min, $sec;
##	 print "Connection: close\n";
      print "\n";

      my $imgfile;

      if ($targetgroup eq 'ads')
      {
         $imgfile = /usr/share/ufdbGuard;
         $imgfile = /usr/share/ufdbGuard;
         $imgfile = /usr/share/ufdbGuard;
         $imgfile = /usr/share/ufdbGuard;
      }
      else
      {
	 if ($mode eq 'cross') {
	    $imgfile = /usr/share/ufdbGuard;
	 }
	 elsif ($mode eq 'square') {
	    $imgfile = /usr/share/ufdbGuard;
	 }
	 elsif ($mode eq 'simple-red'  ||  $mode eq 'transparent'  ||  $mode eq 'transparant') {
	    $imgfile = /usr/share/ufdbGuard;
	 }
	 else  {
	    $imgfile = /usr/share/ufdbGuard;
	 }
      }
      $root = /usr/share/ufdbGuard;
      open( BLOCKEDPNG, "$root/images/$imgfile" )  ||  print "failed to open $root/images/$imgfile\n";
      print $buffer while (read (BLOCKEDPNG,$buffer,8192));
      close( BLOCKEDPNG );
   }
   elsif ($contentType == CT_STREAM)
   {
      print "Content-Type: application/octet-stream\n";
      ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = /usr/share/ufdbGuard;
      printf "Expires: %s, %02d-%s-%04d %02d:%02d:%02d GMT\n", 
	     $day[$wday], $mday, $month[$mon], $year+1900, $hour, $min, $sec;
##	 print "Connection: close\n";
      print "\n";
   }
   elsif ($contentType == CT_JAVA)
   {
      print "Content-Type: application/x-javascript\n";
      ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = /usr/share/ufdbGuard;
      printf "Expires: %s, %02d-%s-%04d %02d:%02d:%02d GMT\n", 
	     $day[$wday], $mday, $month[$mon], $year+1900, $hour, $min, $sec;
##	 print "Connection: close\n";
      print "\n";

      print "\n";
   }
   elsif ($contentType == CT_JSON)
   {
      print "Content-Type: application/json\n";
      ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = /usr/share/ufdbGuard;
      printf "Expires: %s, %02d-%s-%04d %02d:%02d:%02d GMT\n", 
	     $day[$wday], $mday, $month[$mon], $year+1900, $hour, $min, $sec;
##	 print "Connection: close\n";
      print "\n";

      print "\n";
   }
   elsif ($contentType == CT_CSS)
   {
      print "Content-Type: text/css\n";
      ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = /usr/share/ufdbGuard;
      printf "Expires: %s, %02d-%s-%04d %02d:%02d:%02d GMT\n", 
	     $day[$wday], $mday, $month[$mon], $year+1900, $hour, $min, $sec;
##	 print "Connection: close\n";
      print "\n";

      print "\n";
   }
   elsif ($contentType == CT_TEXT)
   {
      print "Content-Type: text/plain\n";
      ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = /usr/share/ufdbGuard;
      printf "Expires: %s, %02d-%s-%04d %02d:%02d:%02d GMT\n", 
	     $day[$wday], $mday, $month[$mon], $year+1900, $hour, $min, $sec;
##	 print "Connection: close\n";
      print "\n";

      print "\n";
   }
   elsif ($contentType == CT_XML)
   {
      print "Content-Type: text/xml\n";
      ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = /usr/share/ufdbGuard;
      printf "Expires: %s, %02d-%s-%04d %02d:%02d:%02d GMT\n", 
	     $day[$wday], $mday, $month[$mon], $year+1900, $hour, $min, $sec;
##	 print "Connection: close\n";
      print "\n";

      print "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n";
      print "<ufdbguardd>\n";
      print "   <dummy value=\"0\" />\n";
      print "</ufdbguardd>\n";
   }
   else 	# CT_HTML
   {
      if ($targetgroup eq 'fatal-error')
      {
	 print "Content-Type: text/html\n";
	 ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = /usr/share/ufdbGuard;
	 printf "Expires: %s, %02d-%s-%04d %02d:%02d:%02d GMT\n", 
		$day[$wday], $mday, $month[$mon], $year+1900, $hour, $min, $sec;
	 print "Content-Language: $lang\n";
##	 print "Connection: close\n";
	 print "\n";

	 print "<html>";
	 print "<head><title>The URL filter has a fatal error</title></head>\n";
	 print "<body bgcolor=\"#e0e0e0\">\n";
	 print "<center>\n";
	 print "<font color=red><b>\n" .
	       "Access to the internet is blocked because<br>\n" .
	       "the URL filter has a fatal error. <br>\n" . 
	       "Ask your helpdesk or web proxy administrator for assistance." .
	       "</b></font>\n";
	 print "</center>\n";
	 print "$html_comment";
	 print "</body>\n";
	 print "</html>\n";
      }
      elsif ($targetgroup eq 'loading-database')
      {
	 print "Content-Type: text/html\n";
	 ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = /usr/share/ufdbGuard;
	 printf "Expires: %s, %02d-%s-%04d %02d:%02d:%02d GMT\n", 
		$day[$wday], $mday, $month[$mon], $year+1900, $hour, $min, $sec;
	 print "Content-Language: $lang\n";
##	 print "Connection: close\n";
	 print "\n";

	 print "<html>";
	 print "<head><title>a new URL database is being loaded</title></head>\n";
	 print "<body bgcolor=\"#e0e0e0\">\n";
	 print "<center>\n";
	 print "<font color=red><b>\n" .
	       "Access to the internet is temporarily blocked because<br>\n" .
	       "a new URL database is being loaded by the URL filter. <br>\n" . 
	       "Wait one minute and try again." .
	       "</b></font>\n";
	 print "</center>\n";
	 print "$html_comment";
	 print "</body>\n";
	 print "</html>\n";
      }
      else
      {
	 if ($lang eq 'de') {
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	 }
	 elsif ($lang eq 'pl') {
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	 }
	 elsif ($lang eq 'sv') {
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	 }
	 elsif ($lang eq 'nl') {
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	 }
	 elsif ($lang eq 'es') {
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	 }
	 elsif ($lang eq 'it') {
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	 }
	 elsif ($lang eq 'pt') {
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	 }
	 elsif ($lang eq 'fr') {
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	 }
	 elsif ($lang eq 'tr') {
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	 }
	 elsif ($lang eq 'NEWLANGUAGE') {
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	 }
	 else {   # default (matches 'en')
	    $forbidden = /usr/share/ufdbGuard;
	    $title = /usr/share/ufdbGuard;
	    $explanation_prefix = /usr/share/ufdbGuard;
	    $explanation_suffix = /usr/share/ufdbGuard;
	    $more_info = /usr/share/ufdbGuard;
	    $go_back = /usr/share/ufdbGuard;
	    $lang = /usr/share/ufdbGuard;
	 }

	 if ($color eq 'orange')
	 {
	    $textcolor = /usr/share/ufdbGuard;
	    $bgcolor = /usr/share/ufdbGuard;
	 }
	 elsif ($color eq 'white')
	 {
	    $textcolor = /usr/share/ufdbGuard;
	    $bgcolor = /usr/share/ufdbGuard;
	 }
	 elsif ($color eq 'black')
	 {
	    $textcolor = /usr/share/ufdbGuard;
	    $bgcolor = /usr/share/ufdbGuard;
	 }
	 elsif ($color eq 'red')
	 {
	    $textcolor = /usr/share/ufdbGuard;
	    $bgcolor = /usr/share/ufdbGuard;
	 }
	 elsif ($color eq 'grey'  ||  $color eq 'gray')
	 {
	    $textcolor = /usr/share/ufdbGuard;
	    $bgcolor = /usr/share/ufdbGuard;
	 }
	 else	# default color: orange
	 {
	    $textcolor = /usr/share/ufdbGuard;
	    $bgcolor = /usr/share/ufdbGuard;
	 }

	 if ($size eq 'normal')
	 {
	    $titlesize = /usr/share/ufdbGuard;
	    $textsize = /usr/share/ufdbGuard;
	 }
	 elsif ($size eq 'small')
	 {
	    $titlesize = /usr/share/ufdbGuard;
	    $textsize = /usr/share/ufdbGuard;
	 }
	 elsif ($size eq 'large')
	 {
	    $titlesize = /usr/share/ufdbGuard;
	    $textsize = /usr/share/ufdbGuard;
	 }
	 else    # default size: normal
	 {
	    $titlesize = /usr/share/ufdbGuard;
	    $textsize = /usr/share/ufdbGuard;
	    $size = /usr/share/ufdbGuard;
	 }

	 $url =~ s/[?;&].*//;

	 print "Content-Type: text/html\n";
	 ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = /usr/share/ufdbGuard;
	 printf "Expires: %s, %02d-%s-%04d %02d:%02d:%02d GMT\n", 
		$day[$wday], $mday, $month[$mon], $year+1900, $hour, $min, $sec;
	 print "Content-Language: $lang\n";
##	 print "Connection: close\n";
	 print "\n";

	 if ($targetgroup eq 'ads')
	 {
	    my $text;
	    $text = /usr/share/ufdbGuard;	 # transparent
	    $text = /usr/share/ufdbGuard;
	    $text = /usr/share/ufdbGuard;
	    $text = /usr/share/ufdbGuard;
	    $text = /usr/share/ufdbGuard;

	    print "<html>\n";
	    print "<head><title>$title</title></head>\n";
	    print "<body>\n";
	    print "<font size=\"$textsize\">$text</font>\n";
	    print "$html_comment";
	    print "</body>\n";
	    print "</html>\n";
	 }
	 else
	 {
	    if ($mode eq 'simple-red')
	    {
	       my $whyblocked = /usr/share/ufdbGuard;
	       print "<html>";
	       print "<head><title>$title</title></head>\n";
	       print "<body bgcolor=\"ffcccc\" link=\"red\" alink=\"red\" vlink=\"red\" text=\"red\">\n";
	       print "<p align=center>\n";
	       print "<a title=\"$whyblocked\">$forbidden<br><i>$targetgroup</i></a>\n";
	       print "$html_comment";
	       print "</body>\n";
	       print "</html>\n";
	    }
	    else
	    {
	       print "<html>";
	       print "<head> <title>$title</title> </head>\n";
	       print "<body bgcolor=\"$bgcolor\" text=\"$textcolor\">\n";
	       print "<font size=\"$titlesize\">$forbidden</font> <br>\n";
	       print "<font size=\"$textsize\">\n";
	       print "$explanation_prefix <i>$targetgroup</i> $explanation_suffix <br>\n";
	       print "URL: $url <br>\n";
	       print "<p>\n";
	       print "<a href=\"javascript:history.go(-1);\">$go_back</a>. <br>\n";
	       print "$admin\n";
	       print "<p>\n";
	       print "$more_info\n";
	       print "</font>\n";
	       print "<br>\n&nbsp;<p />\n";
	       print "<font size=\"-3\">";
	       print "user=$clientuser &nbsp; "      if (defined($clientuser)  &&  length($clientuser)>0);
	       print "client=$clientaddr &nbsp; "    if (defined($clientaddr)  &&  length($clientaddr)>0);
	       print "group=$clientgroup &nbsp; "    if (defined($clientgroup)  &&  length($clientgroup)>0);
	       print "source=$clientname &nbsp; "    if (defined($clientname)  &&  length($clientname)>0);
	       print "</font>\n";
	       print "$html_comment";
	       print "</body>\n";
	       print "</html>\n";
	    }
	 }
      }
   }
}


exit 0;


